@@ -1,7 +1,7 @@
 /* 
  * Seahorse
  * 
- * Copyright (C) 2005 Nate Nielsen 
+ * Copyright (C) 2005 Stefan Walter
  * 
  * This program is free software; you can redistribute it and/or modify 
  * it under the terms of the GNU Lesser General Public License as
@@ -24,6 +24,7 @@
 #include <string.h>
 
 #include "cryptui-key-store.h"
+#include "cryptui-priv.h"
 
 enum {
     PROP_0,
@@ -31,6 +32,7 @@ enum {
     PROP_MODE,
     PROP_SEARCH,
     PROP_USE_CHECKS,
+    PROP_SORTABLE,
     PROP_NONE_OPTION
 };
 
@@ -62,6 +64,7 @@ struct _CryptUIKeyStorePriv {
     gpointer                    filter_data;
     
     gboolean                    use_checks;
+    gboolean                    sortable;
     gchar                       *none_option;
 };
 
@@ -104,7 +107,15 @@ key_store_row_add (CryptUIKeyStore *ckstore, const gchar *key, GtkTreeIter *iter
     
     /* Do we already have a row for this key? */
     ref = (GtkTreeRowReference*)g_hash_table_lookup (ckstore->priv->rows, key);
-    g_return_if_fail (ref == NULL);
+    if (ref != NULL) {
+        
+        /* If so return the previous reference */
+        path = gtk_tree_row_reference_get_path (ref);
+        if (path != NULL) {
+            gtk_tree_model_get_iter (GTK_TREE_MODEL (ckstore->priv->store), iter, path);
+            return;
+        }
+    }
 
     gtk_tree_store_append (GTK_TREE_STORE (ckstore->priv->store), iter, NULL);
     path = gtk_tree_model_get_path (GTK_TREE_MODEL (ckstore->priv->store), iter);
@@ -157,13 +168,20 @@ key_store_key_changed (CryptUIKeyset *ckset, const gchar *key,
     GtkTreeIter iter;
     GtkTreePath *path;
 
-    g_assert (ref != NULL);
-    
-    path = gtk_tree_row_reference_get_path (ref);
-    if (path) {
-        if (gtk_tree_model_get_iter (GTK_TREE_MODEL (ckstore->priv->store), &iter, path))
-            key_store_set (ckstore, key, &iter);
-        gtk_tree_path_free (path);
+    /* If never seen before treat as added */
+    if (ref == NULL) {
+        g_return_if_fail (!g_hash_table_lookup (ckstore->priv->rows, key));
+        key_store_row_add (ckstore, key, &iter);
+        key_store_set (ckstore, key, &iter);
+        
+    /* Otherwise just update */
+    } else {
+        path = gtk_tree_row_reference_get_path (ref);
+        if (path) {
+            if (gtk_tree_model_get_iter (GTK_TREE_MODEL (ckstore->priv->store), &iter, path))
+                key_store_set (ckstore, key, &iter);
+            gtk_tree_path_free (path);
+        }
     }
 }
 
@@ -174,7 +192,7 @@ key_store_key_removed (CryptUIKeyset *ckset, const gchar *key,
     GtkTreeIter iter;
     GtkTreePath *path;
     
-    g_assert (ref != NULL);
+    g_return_if_fail (ref != NULL);
     
     path = gtk_tree_row_reference_get_path (ref);
     if (path) {
@@ -400,6 +418,66 @@ sort_default_comparator (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b,
     
 }
 
+static void 
+load_sort_order (CryptUIKeyStore *ckstore)
+{
+    GtkSortType ord = GTK_SORT_ASCENDING;
+    gchar *sort = NULL;
+    const gchar *name;
+
+    if (ckstore->priv->sortable)
+        sort = _cryptui_gconf_get_string (SEAHORSE_RECIPIENTS_SORT_KEY);
+    
+    name = sort ? sort : "";
+    
+    /* Prefix with a minus means descending */
+    if (name[0] == '-') {
+        ord = GTK_SORT_DESCENDING;
+        name++;
+    }
+    
+    /* Prefix with a plus meaning ascending */
+    else if (name[0] == '+') {
+        ord = GTK_SORT_ASCENDING;
+        name++;
+    }
+    
+    if (g_ascii_strcasecmp ("name", name) == 0)
+        gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (ckstore), 
+                                              CRYPTUI_KEY_STORE_NAME, ord);
+    else if (g_ascii_strcasecmp ("id", name) == 0)
+        gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (ckstore), 
+                                              CRYPTUI_KEY_STORE_KEYID, ord);
+    else 
+        gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (ckstore), 
+                                              GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, ord);
+    
+    g_free (sort);
+}
+
+static void
+sort_changed (GtkTreeSortable *sort, CryptUIKeyStore *ckstore)
+{
+    gint id;
+    GtkSortType ord;
+    const gchar *name = NULL;
+    gchar *x;
+    
+    if (!ckstore->priv->sortable)
+        return;
+
+    if (gtk_tree_sortable_get_sort_column_id (sort, &id, &ord)) {
+        if (id == CRYPTUI_KEY_STORE_NAME)
+            name = "name";
+        else if (id == CRYPTUI_KEY_STORE_KEYID)
+            name = "id";
+    }
+    
+    x = g_strconcat (ord == GTK_SORT_DESCENDING ? "-" : "", name ? name : "", NULL);
+    _cryptui_gconf_set_string (SEAHORSE_RECIPIENTS_SORT_KEY, x);
+    g_free (x);
+}
+
 /* -----------------------------------------------------------------------------
  * OBJECT 
  */
@@ -425,7 +503,9 @@ cryptui_key_store_init (CryptUIKeyStore *ckstore)
 static GObject*  
 cryptui_key_store_constructor (GType type, guint n_props, GObjectConstructParam* props)
 {
-    GObject *obj = G_OBJECT_CLASS (cryptui_key_store_parent_class)->constructor (type, n_props, props);
+    GObject *obj;
+    
+    obj = G_OBJECT_CLASS (cryptui_key_store_parent_class)->constructor (type, n_props, props);
     CryptUIKeyStore *ckstore = CRYPTUI_KEY_STORE (obj);
 
     /* Hookup to the current object */
@@ -433,6 +513,7 @@ cryptui_key_store_constructor (GType type, guint n_props, GObjectConstructParam*
 
     /* Default sort function */
     gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (ckstore), sort_default_comparator, NULL, NULL);
+    g_signal_connect (ckstore, "sort-column-changed", G_CALLBACK (sort_changed), ckstore);
     
     ckstore->priv->initialized = TRUE;
     key_store_populate (ckstore);
@@ -515,8 +596,14 @@ cryptui_key_store_set_property (GObject *gobject, guint prop_id,
     /* The search text */
     case PROP_SEARCH:
         cryptui_key_store_set_search_text (ckstore, g_value_get_string (value));
-        break; 
+        break;
+
+    /* Sortable columns */
+    case PROP_SORTABLE:
+        ckstore->priv->sortable = g_value_get_boolean (value);
+        break;
     
+    /* Display and use the check boxes */
     case PROP_USE_CHECKS:
         ckstore->priv->use_checks = g_value_get_boolean (value);
         break;
@@ -557,6 +644,11 @@ cryptui_key_store_get_property (GObject *gobject, guint prop_id,
             ckstore->priv->filter_mode == CRYPTUI_KEY_STORE_MODE_RESULTS ? ckstore->priv->search_text : "");
         break;
     
+    case PROP_SORTABLE:
+        g_value_set_boolean (value, ckstore->priv->sortable);
+        load_sort_order (ckstore);
+        break;
+    
     case PROP_USE_CHECKS:
         g_value_set_boolean (value, ckstore->priv->use_checks);
         break;
@@ -596,6 +688,10 @@ cryptui_key_store_class_init (CryptUIKeyStoreClass *klass)
         g_param_spec_string ("search", "Search text", "Key store search text for when in results mode",
                              "", G_PARAM_READWRITE));
                              
+    g_object_class_install_property (gobject_class, PROP_SORTABLE,
+        g_param_spec_boolean ("sortable", "User sortable columns", "Allow user to sort columns",
+                              FALSE, G_PARAM_READWRITE));
+
     g_object_class_install_property (gobject_class, PROP_USE_CHECKS,
         g_param_spec_boolean ("use-checks", "Use check boxes", "Use check box column to denote selection",
                               FALSE, G_PARAM_READWRITE));
@@ -618,6 +714,26 @@ cryptui_key_store_new (CryptUIKeyset *keyset, gboolean use_checks,
     return (CryptUIKeyStore*)obj;
 }
 
+CryptUIKeyset*
+cryptui_key_store_get_keyset (CryptUIKeyStore *ckstore)
+{
+    return ckstore->ckset;
+}
+
+void
+cryptui_key_store_set_sortable (CryptUIKeyStore *ckstore, gboolean sortable)
+{
+    g_object_set (ckstore, "sortable", sortable, NULL);
+}
+
+gboolean 
+cryptui_key_store_get_sortable (CryptUIKeyStore *ckstore)
+{
+    gboolean sortable;
+    g_object_get (ckstore, "sortable", &sortable, NULL);
+    return sortable;
+}
+
 void
 cryptui_key_store_check_toggled (CryptUIKeyStore *ckstore, GtkTreeView *view, GtkTreeIter *iter)
 {
@@ -655,7 +771,7 @@ cryptui_key_store_get_iter_from_key (CryptUIKeyStore *ckstore, const gchar *key,
     g_return_val_if_fail (CRYPTUI_IS_KEY_STORE (ckstore), FALSE);
     g_return_val_if_fail (iter != NULL, FALSE);
     
-    if (key == NULL) {
+    if (key == NULL || !key[0]) {
         /* The none option for NULL */
         if (ckstore->priv->none_option) {
             if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (ckstore->priv->store), &base)) {
@@ -719,7 +835,7 @@ cryptui_key_store_have_selected_keys (CryptUIKeyStore *ckstore, GtkTreeView *vie
     g_return_val_if_fail (GTK_IS_TREE_VIEW (view), FALSE);
     
     if (ckstore->priv->use_checks) {
-        GtkTreeModel* model = GTK_TREE_MODEL (ckstore);
+        GtkTreeModel* model = GTK_TREE_MODEL (ckstore->priv->store);
         GtkTreeIter iter;
         gboolean check;
             
@@ -731,13 +847,13 @@ cryptui_key_store_have_selected_keys (CryptUIKeyStore *ckstore, GtkTreeView *vie
                     return TRUE;
             } while (gtk_tree_model_iter_next (model, &iter));
         }
+    } else {
+        /* Fall back if none checked, or not using checks */
+        selection = gtk_tree_view_get_selection (view);
+        if (gtk_tree_selection_count_selected_rows (selection))
+            return TRUE;
     }
     
-    /* Fall back if none checked, or not using checks */
-    selection = gtk_tree_view_get_selection (view);
-    if (gtk_tree_selection_count_selected_rows (selection))
-        return TRUE;
-    
     return FALSE;
 }
 
@@ -750,7 +866,7 @@ cryptui_key_store_get_selected_keys (CryptUIKeyStore *ckstore, GtkTreeView *view
     g_return_val_if_fail (GTK_IS_TREE_VIEW (view), NULL);
     
     if (ckstore->priv->use_checks) {
-        GtkTreeModel* model = GTK_TREE_MODEL (ckstore);
+        GtkTreeModel* model = GTK_TREE_MODEL (ckstore->priv->store);
         GtkTreeIter iter;
         gboolean check;
             
