@@ -2,7 +2,7 @@
  * Seahorse
  *
  * Copyright (C) 2003 Jacob Perkins
- * Copyright (C) 2005 Nate Nielsen
+ * Copyright (C) 2005 Stefan Walter
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -61,6 +61,7 @@ struct _SeahorseWidget {
     
     /*< private >*/
     GtkUIManager   *ui;
+    gboolean destroying;
 };
 
 struct _SeahorseWidgetClass {
@@ -69,9 +70,13 @@ struct _SeahorseWidgetClass {
 
 GType            seahorse_widget_get_type ();
 
-SeahorseWidget*  seahorse_widget_new                (gchar            *name);
+SeahorseWidget*  seahorse_widget_new                (const gchar      *name,
+                                                     GtkWindow        *parent);
 
-SeahorseWidget*  seahorse_widget_new_allow_multiple (gchar            *name);
+SeahorseWidget*  seahorse_widget_new_allow_multiple (const gchar      *name,
+                                                     GtkWindow        *parent);
+
+SeahorseWidget*  seahorse_widget_find               (const gchar      *name);
 
 GtkWidget*       seahorse_widget_get_top            (SeahorseWidget   *swidget);
 
