@@ -2,7 +2,7 @@
  * Seahorse
  *
  * Copyright (C) 2003 Jacob Perkins
- * Copyright (C) 2004, 2005, 2006 Nate Nielsen
+ * Copyright (C) 2004, 2005, 2006 Stefan Walter
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -318,7 +318,7 @@ update_key_row (SeahorseKeyManagerStore *skstore, SeahorseKey *skey, guint uid,
                 expires_date = G_MAXLONG;
             }
             else
-                expires = seahorse_util_get_date_string (expires_date);
+                expires = seahorse_util_get_display_date_string (expires_date);
         }
         
         /* Only differentiate if the view shows more than one type of key */
